/**
 * An interface to represent a functional iterator
 * 
 * @author Viera K. Proulx
 */
public interface Traversal<E>{

    /** 
     * produce a traversal for the rest of the dataset
     * throw an exception if the dataset is empty
     * 
     * @return new traversal advanced to the next element
     * @throws IllegalUseOfTraversalException if the dataset is empty
     */
    public Traversal<E> getRest() throws IllegalUseOfTraversalException; 
    
    /** 
     * provide the first element of the dataset 
     * 
     * @return the first element in the dataset
     * @throws IllegalUseOfTraversalException if the dataset is empty
     */
    public E getFirst() throws IllegalUseOfTraversalException; 
    
    /** 
     * test whether the dataset has no elements 
     * 
     * @return true if dataset is empty
     */
    public boolean isEmpty(); 
}

